/*
 PM - Symphony of Lights
 */

#include <avr/io.h>
#include <util/delay.h>

#define SHORT_DELAY 25
#define MEDIUM_DELAY 50
#define LONG_DELAY 75

/*
Stinge intreg cubul
*/
void clearCube() {

	PORTD &= ~(1 << PD0);
	PORTD &= ~(1 << PD1);
	PORTD &= ~(1 << PD4);
	PORTD &= ~(1 << PD5);

	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTA &= ~(1 << PA2);
	PORTA &= ~(1 << PA3);
	PORTA &= ~(1 << PA4);
	PORTA &= ~(1 << PA5);
	PORTA &= ~(1 << PA6);
	PORTA &= ~(1 << PA7);

	PORTC &= ~(1 << PC0);
	PORTC &= ~(1 << PC1);
	PORTC &= ~(1 << PC2);
	PORTC &= ~(1 << PC3);
	PORTC &= ~(1 << PC4);
	PORTC &= ~(1 << PC5);
	PORTC &= ~(1 << PC6);
	PORTC &= ~(1 << PC7);

}

/*
Genereaza pulsatia unei coloane de leduri primita ca parametru, 
pentru coloanele de pe PORTC.
*/
void flickerColumnPortC(unsigned int column) {

	PORTD |= (1 << PD5);
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD1);

	PORTC |= (1 << column);
	_delay_ms(MEDIUM_DELAY);
	PORTC &= ~(1 << column);
	
	PORTD &= ~(1 << PD5);
	PORTD &= ~(1 << PD0);
	PORTD &= ~(1 << PD4);
	PORTD &= ~(1 << PD1);

	_delay_ms(MEDIUM_DELAY);

}

/*
Genereaza pulsatia unei coloane de leduri primita ca parametru, 
pentru coloanele de pe PORTA
*/
void flickerColumnPortA(unsigned int column) {

	PORTD |= (1 << PD5);
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD1);

	PORTA |= (1 << column);
	_delay_ms(MEDIUM_DELAY);
	PORTA &= ~(1 << column);

	PORTD &= ~(1 << PD5);
	PORTD &= ~(1 << PD0);
	PORTD &= ~(1 << PD4);
	PORTD &= ~(1 << PD1);
	
	_delay_ms(MEDIUM_DELAY);

}

/*
Genereaza aprinderea unei coloane de leduri primita ca parametru, 
pentru coloanele de pe PORTC
*/
void lightColumnPortC(unsigned int column) {

	clearCube();

	PORTD |= (1 << PD5);
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD1);

	PORTC |= (1 << column);
	_delay_ms(MEDIUM_DELAY);

}

/*
Genereaza aprinderea unei coloane de leduri primita ca parametru, 
pentru coloanele de pe PORTA
*/
void lightColumnPortA(unsigned int column) {

	clearCube();

	PORTD |= (1 << PD5);
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD1);

	PORTA |= (1 << column);
	_delay_ms(MEDIUM_DELAY);

}

/*
Genereaza pulsatia unui led pe baza randului si coloanei primite 
ca parametru, pentru coloanele de pe PORTC
*/
void flickerLedPortC(unsigned int row, unsigned int column) {

	PORTD |= (1 << row);
	PORTC |= (1 << column);
	_delay_ms(SHORT_DELAY);
	PORTC &= ~(1 << column);
	PORTD &= ~(1 << row);
	_delay_ms(SHORT_DELAY);

}

/*
Genereaza aprinderea unui led pe baza randului si coloanei 
primite ca parametru, pentru coloanele de pe PORTA
*/
void lightLedPortA(unsigned int row, unsigned int column) {

	_delay_ms(MEDIUM_DELAY);
	PORTD |= (1 << row);
	PORTA |= (1 << column);

}

/*
Genereaza aprinderea unui led pe baza randului si coloanei 
primite ca parametru, pentru coloanele de pe PORTC
*/
void lightLedPortC(unsigned int row, unsigned int column) {

	_delay_ms(MEDIUM_DELAY);
	PORTD |= (1 << row);
	PORTC |= (1 << column);

}

/*
Genereaza pulsatia unui led pe baza randului si coloanei primite 
ca parametru, pentru coloanele de pe PORTA
*/
void flickerLedPortA(unsigned int row, unsigned int column) {

	PORTD |= (1 << row);
	PORTA |= (1 << column);
	_delay_ms(SHORT_DELAY);
	PORTA &= ~(1 << column);
	PORTD &= ~(1 << row);
	_delay_ms(SHORT_DELAY);

}

/*
Aprinde toate coloanele cubului
*/
void lightAllColumns() {

	PORTA |= (1 << PA0);
	PORTA |= (1 << PA1);
	PORTA |= (1 << PA2);
	PORTA |= (1 << PA3);
	PORTA |= (1 << PA4);
	PORTA |= (1 << PA5);
	PORTA |= (1 << PA6);
	PORTA |= (1 << PA7);

	PORTC |= (1 << PC0);
	PORTC |= (1 << PC1);
	PORTC |= (1 << PC2);
	PORTC |= (1 << PC3);
	PORTC |= (1 << PC4);
	PORTC |= (1 << PC5);
	PORTC |= (1 << PC6);
	PORTC |= (1 << PC7);

}

/*
Aprinde toate liniile cubului
*/
void lightAllRows() {

	PORTD |= (1<< PD5);
	PORTD |= (1<< PD0);
	PORTD |= (1<< PD4);
	PORTD |= (1<< PD1);

}

/*
Stinge toate randurile cubului
*/
void offAllRows() {

	PORTD &= ~(1<< PD5);
	PORTD &= ~(1<< PD0);
	PORTD &= ~(1<< PD4);
	PORTD &= ~(1<< PD1);

}

/*
Ledurile pulseaza pe fiecare rand in parte, 
de la baza spre partea superioara, generand 
iluzia unei miscari ascendente
*/
void rowsUp(){
		
	clearCube();
	lightAllColumns();

	PORTD |= (1<< PD5);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD5);

	PORTD |= (1<< PD0);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD0);	

	PORTD |= (1<< PD4);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD4);

	PORTD |= (1<< PD1);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD1);
	
}

/*
Ledurile pulseaza pe fiecare rand in parte, 
de la partea superioara spre baza, generand 
iluzia unei miscari descentente
*/
void rowsDown(){
		
	clearCube();
	lightAllColumns();
	
	PORTD |= (1<< PD1);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD1);

	PORTD |= (1<< PD4);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD4);

	PORTD |= (1<< PD0);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD0);

	PORTD |= (1<< PD5);
	_delay_ms(LONG_DELAY);
	PORTD &= ~(1 << PD5);

}

/*
Ledurile cubului se aprind nivel cu nivel, pentru 
ca la final intregul cub sa fie aprins, generand 
astfel un efect gradual
*/
void fill(){

	clearCube();
	lightAllColumns();

	_delay_ms(LONG_DELAY);	

	PORTD |= (1<< PD5);
	_delay_ms(LONG_DELAY);

	PORTD |= (1<< PD0);
	_delay_ms(LONG_DELAY);	

	PORTD |= (1<< PD4);
	_delay_ms(LONG_DELAY);

	PORTD |= (1<< PD1);
	_delay_ms(LONG_DELAY);

	PORTD &= ~(1 << PD1);
	_delay_ms(LONG_DELAY);

	PORTD &= ~(1 << PD4);
	_delay_ms(LONG_DELAY);

	PORTD &= ~(1 << PD0);
	_delay_ms(LONG_DELAY);	

	PORTD &= ~(1 << PD5);
	_delay_ms(LONG_DELAY);

}

/*
Toate ledurile cubului pulseaza intermitent 
la intervale regulate de timp
*/
void flicker(){
		
	clearCube();
	lightAllColumns();
	lightAllRows();
	
	_delay_ms(100);

	offAllRows();

	_delay_ms(100);

}

/*
Fluctuatiile ledurilor formeaza un “vartej”, din 
exteriorul cubului catre centru, pentru ca apoi 
sa revina in exteriorul cubului
*/
void vortex(){

	clearCube();

	lightColumnPortC(PC0);
	lightColumnPortC(PC4);
	lightColumnPortA(PA7);
	lightColumnPortA(PA3);

	lightColumnPortA(PA2);
	lightColumnPortA(PA1);
	lightColumnPortA(PA0);
	lightColumnPortA(PA4);

	lightColumnPortC(PC7);
	lightColumnPortC(PC3);
	lightColumnPortC(PC2);
	lightColumnPortC(PC1);

	lightColumnPortC(PC5);
	lightColumnPortA(PA6);
	lightColumnPortA(PA5);
	lightColumnPortC(PC6);

	lightColumnPortC(PC5);
	lightColumnPortA(PA6);

	lightColumnPortA(PA2);
	lightColumnPortA(PA1);
	lightColumnPortA(PA0);
	lightColumnPortA(PA4);

	lightColumnPortC(PC7);
	lightColumnPortC(PC3);
	lightColumnPortC(PC2);
	lightColumnPortC(PC1);

}

/*
Ledurile pulseaza aleator si cu mare viteza
*/
void randomLeds() {

	clearCube();

	flickerLedPortC(PD0, PC7);
	flickerLedPortA(PD5, PA5);
	flickerLedPortC(PD1, PC0);
	flickerLedPortA(PD4, PA4);
	flickerLedPortC(PD1, PC2);
	flickerLedPortA(PD0, PA7);
	flickerLedPortC(PD4, PC4);
	flickerLedPortA(PD5, PA2);
	flickerLedPortC(PD0, PC1);
	flickerLedPortA(PD4, PA6);
	flickerLedPortC(PD1, PC6);
	flickerLedPortA(PD5, PA3);
	flickerLedPortC(PD5, PC3);
	flickerLedPortA(PD0, PA0);
	flickerLedPortC(PD4, PC5);
	flickerLedPortA(PD1, PA1);

	flickerLedPortC(PD1, PC7);
	flickerLedPortA(PD0, PA5);
	flickerLedPortC(PD4, PC0);
	flickerLedPortA(PD5, PA4);
	flickerLedPortC(PD4, PC2);
	flickerLedPortA(PD1, PA7);
	flickerLedPortC(PD5, PC4);
	flickerLedPortA(PD0, PA2);
	flickerLedPortC(PD1, PC1);
	flickerLedPortA(PD5, PA6);
	flickerLedPortC(PD4, PC6);
	flickerLedPortA(PD0, PA3);
	flickerLedPortC(PD0, PC3);
	flickerLedPortA(PD4, PA0);
	flickerLedPortC(PD5, PC5);
	flickerLedPortA(PD4, PA1);

	flickerLedPortC(PD4, PC7);
	flickerLedPortA(PD1, PA5);
	flickerLedPortC(PD5, PC0);
	flickerLedPortA(PD0, PA4);
	flickerLedPortC(PD5, PC2);
	flickerLedPortA(PD4, PA7);
	flickerLedPortC(PD0, PC4);
	flickerLedPortA(PD1, PA2);
	flickerLedPortC(PD4, PC1);
	flickerLedPortA(PD0, PA6);
	flickerLedPortC(PD5, PC6);
	flickerLedPortA(PD1, PA3);
	flickerLedPortC(PD1, PC3);
	flickerLedPortA(PD5, PA0);
	flickerLedPortC(PD0, PC5);
	flickerLedPortA(PD5, PA1);

	flickerLedPortC(PD5, PC7);
	flickerLedPortA(PD3, PA5);
	flickerLedPortC(PD0, PC0);
	flickerLedPortA(PD1, PA4);
	flickerLedPortC(PD0, PC2);
	flickerLedPortA(PD5, PA7);
	flickerLedPortC(PD1, PC4);
	flickerLedPortA(PD4, PA2);
	flickerLedPortC(PD5, PC1);
	flickerLedPortA(PD1, PA6);
	flickerLedPortC(PD0, PC6);
	flickerLedPortA(PD4, PA3);
	flickerLedPortC(PD4, PC3);
	flickerLedPortA(PD0, PA0);
	flickerLedPortC(PD1, PC5);
	flickerLedPortA(PD0, PA1);

}

/*
Cubul se aprinde cu fiecare led in parte, 
in mod aleator, pentru ca la final cubul sa ramana aprins
*/
void allCube() {

	clearCube();

	lightLedPortC(PD0, PC7);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA5);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC0);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA4);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC2);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA7);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC4);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA2);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC1);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA6);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC6);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA3);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC3);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA0);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC5);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA1);
	_delay_ms(LONG_DELAY);	

	lightLedPortC(PD1, PC7);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA5);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC0);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA4);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC2);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA7);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC4);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA2);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC1);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA6);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC6);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA3);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC3);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA0);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC5);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA1);
	_delay_ms(LONG_DELAY);	

	lightLedPortC(PD4, PC7);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA5);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC0);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA4);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC2);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA7);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC4);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA2);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC1);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA6);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC6);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA3);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC3);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA0);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC5);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA1);
	_delay_ms(LONG_DELAY);	

	lightLedPortC(PD5, PC7);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD3, PA5);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC0);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA4);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC2);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD5, PA7);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC4);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA2);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD5, PC1);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD1, PA6);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD0, PC6);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD4, PA3);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD4, PC3);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA0);
	_delay_ms(LONG_DELAY);	
	lightLedPortC(PD1, PC5);
	_delay_ms(LONG_DELAY);	
	lightLedPortA(PD0, PA1);
	_delay_ms(LONG_DELAY);	

}

/*
Coloanele cubului sunt aprinse aleator la 
intervale regulate de timp, pentru crearea 
unui efect vizual aparte
*/
void randomColumns(){

	clearCube();
	
	flickerColumnPortC(PC1);
	flickerColumnPortA(PA4);
	flickerColumnPortC(PC3);
	flickerColumnPortA(PA2);

	flickerColumnPortC(PC0);
	flickerColumnPortA(PA6);
	flickerColumnPortC(PC6);
	flickerColumnPortA(PA0);

	flickerColumnPortC(PC2);
	flickerColumnPortA(PA1);
	flickerColumnPortC(PC7);
	flickerColumnPortA(PA5);

	flickerColumnPortC(PC4);
	flickerColumnPortA(PA3);
	flickerColumnPortC(PC5);
	flickerColumnPortA(PA7);


}

/*
Ledurile pulseaza formand iluzia unor linii 
orizontale aleatoare, generate la intervale 
aleatoare de timp
*/
void lines() {

	clearCube();

	flickerLedPortC(PD1, PC0);
	flickerLedPortC(PD1, PC4);
	flickerLedPortA(PD1, PA7);
	flickerLedPortA(PD1, PA3);

	_delay_ms(3 * LONG_DELAY);

	flickerLedPortA(PD0, PA2);
	flickerLedPortA(PD0, PA6);
	flickerLedPortC(PD0, PC5);
	flickerLedPortC(PD0, PC1);

	_delay_ms(2 * LONG_DELAY);	
	
	flickerLedPortC(PD4, PC2);
	flickerLedPortC(PD4, PC6);
	flickerLedPortA(PD4, PA5);
	flickerLedPortA(PD4, PA1);
	
	_delay_ms(LONG_DELAY);	

	flickerLedPortA(PD5, PA0);
	flickerLedPortA(PD5, PA4);
	flickerLedPortC(PD5, PC7);
	flickerLedPortC(PD5, PC3);

	_delay_ms(3 * LONG_DELAY);

	flickerLedPortC(PD4, PC0);
	flickerLedPortC(PD4, PC4);
	flickerLedPortA(PD4, PA7);
	flickerLedPortA(PD4, PA3);

	_delay_ms(2 * LONG_DELAY);

	flickerLedPortA(PD1, PA2);
	flickerLedPortA(PD1, PA6);
	flickerLedPortC(PD1, PC5);
	flickerLedPortC(PD1, PC1);
	
	_delay_ms(3 * LONG_DELAY);

	flickerLedPortC(PD5, PC2);
	flickerLedPortC(PD5, PC6);
	flickerLedPortA(PD5, PA5);
	flickerLedPortA(PD5, PA1);
	
	_delay_ms(2 * LONG_DELAY);

	flickerLedPortA(PD0, PA0);
	flickerLedPortA(PD0, PA4);
	flickerLedPortC(PD0, PC7);
	flickerLedPortC(PD0, PC3);

	_delay_ms(LONG_DELAY);

	flickerLedPortC(PD5, PC0);
	flickerLedPortC(PD5, PC4);
	flickerLedPortA(PD5, PA7);
	flickerLedPortA(PD5, PA3);

	_delay_ms(LONG_DELAY);	

	flickerLedPortA(PD4, PA2);
	flickerLedPortA(PD4, PA6);
	flickerLedPortC(PD4, PC5);
	flickerLedPortC(PD4, PC1);

	_delay_ms(3 * LONG_DELAY);	
	
	flickerLedPortC(PD0, PC2);
	flickerLedPortC(PD0, PC6);
	flickerLedPortA(PD0, PA5);
	flickerLedPortA(PD0, PA1);

	_delay_ms(3 * LONG_DELAY);	
	
	flickerLedPortA(PD1, PA0);
	flickerLedPortA(PD1, PA4);
	flickerLedPortC(PD1, PC7);
	flickerLedPortC(PD1, PC3);

	_delay_ms(3 * LONG_DELAY);

	flickerLedPortC(PD0, PC0);
	flickerLedPortC(PD0, PC4);
	flickerLedPortA(PD0, PA7);
	flickerLedPortA(PD0, PA3);

	_delay_ms(2 * LONG_DELAY);

	flickerLedPortA(PD5, PA2);
	flickerLedPortA(PD5, PA6);
	flickerLedPortC(PD5, PC5);
	flickerLedPortC(PD5, PC1);
	
	_delay_ms(LONG_DELAY);

	flickerLedPortC(PD1, PC2);
	flickerLedPortC(PD1, PC6);
	flickerLedPortA(PD1, PA5);
	flickerLedPortA(PD1, PA1);
	
	_delay_ms(2 * LONG_DELAY);

	flickerLedPortA(PD4, PA0);
	flickerLedPortA(PD4, PA4);
	flickerLedPortC(PD4, PC7);
	flickerLedPortC(PD4, PC3);

	_delay_ms(3 * LONG_DELAY);

}

int main(void) {
  DDRA = 0xFF;
  DDRC = 0xFF;
  
  DDRD |= (1 << PD0);
  DDRD |= (1 << PD1);
  DDRD |= (1 << PD4);
  DDRD |= (1 << PD5);
  
  int i;

  while (1){

		for(i = 0; i < 10; i++) {
			flicker();
		}

		for(i = 0; i < 2; i++) {
			lines();
		}

		for(i = 0; i < 5; i++) {
			vortex();
		}
		
		for(i = 0; i < 10; i++) {
			rowsUp();
		}

		for(i = 0; i < 5; i++) {
			randomColumns();
		}

		for(i = 0; i < 7; i++) {
			fill();
		}

		for(i = 0; i < 10; i++) {
			rowsDown();
		}
		
		for(i = 0; i < 3; i++) {
			randomLeds();
		}

		allCube();

 }
	
  return 0;

}
